from django.conf.urls import patterns, url


urlpatterns = patterns('mediabrowser.views',
    url('^images/$', 'image_list_view', name='mediabrowser-image-list'),
    url('^images/add/$', 'image_add_view', name='mediabrowser-add-image'),
    
    url('^documents/$', 'document_list_view', name='mediabrowser-document-list'),
    url('^documents/images/$', 'image_as_document_list_view', name='mediabrowser-image-document-list'),
    url('^documents/add/$', 'document_add_view', name='mediabrowser-add-document'),
    
    url('^delete/$', 'asset_delete_view', name='mediabrowser-delete'),
)


from PIL import Image
from django.db import models
from django.conf import settings
import os.path
from .constants import MEDIABROWSER_UPLOAD_TO, ASSET_TYPE_CHOICES

import re

_ = lambda x:x

class Asset(models.Model):
    name = models.TextField(_("name"), max_length=120)
    file = models.FileField(_("file"), upload_to=MEDIABROWSER_UPLOAD_TO)
    type = models.CharField(_("type"), max_length="16", choices=ASSET_TYPE_CHOICES)
    ext = models.CharField(_("type"), max_length=40)
    uploaded_on = models.DateTimeField(_("uploaded on"), auto_now_add=True)
    uploaded_by = models.CharField(_("uploaded by"), max_length=80, blank=True)
    width = models.PositiveIntegerField(_("width"), blank=True, null=True)
    height = models.PositiveIntegerField(_("height"), blank=True, null=True)

    
    def save(self, *args, **kwargs):
        request = kwargs.pop("request", None)
        if request and not self.uploaded_by:
            self.uploaded_by = unicode(request.user)
            
        if not self.name:
            m = re.search(r'([^/]+)$', self.file.url)
            self.name = m.group(1)
            self.name = re.sub(r'(_|-|%20)+', ' ', self.name)
            
        if not self.ext:
            m = re.search(r'(\w+)$', self.name)
            if m:
                self.ext = m.group(1).lower()
            
        if self.ext in ('png','jpg','jpeg','gif'):
            # we treat tiff, bmp and other image types as documents
            self.type = "img"
        else:
            self.type = "doc"
        
        if self.type == "img" and not self.id:
            self.file.open('r+b')
            img = Image.open(self.file)
            (imgW, imgH) = img.size
            max_size = getattr(settings, "MEDIABROWSER_MAX_IMAGE_SIZE", None)
            if max_size:
                if imgW > max_size[0] or imgH > max_size[1]:    
                    ratio = max(float(imgW)/max_size[0], float(imgH)/max_size[1])
                    img = img.resize((int(imgW/ratio), int(imgH/ratio)), Image.ANTIALIAS)
                    self.file.seek(0)
                    img.save(self.file)
            
            self.width, self.height = img.size
        
        super(Asset, self).save(*args, **kwargs)            
    
    def __unicode__(self):
        if self.type == "img":
            return '%s, %d x %d' % (self.name, self.width, self.height)
        return self.name



from django.conf import settings

MEDIABROWSER_UPLOAD_TO = getattr(settings, 'MEDIABROWSER_UPLOAD_TO', "mb/%Y/%m")

MEDIABROWSER_PAGE_SELECTOR_URL = getattr(settings, 'MEDIABROWSER_PAGE_SELECTOR_URL', None)

MEDIABROWSER_USER_PASSES_TEST = getattr(settings, 'MEDIABROWSER_USER_PASSES_TEST',
                                         lambda user:user.is_staff)

ASSET_TYPE_CHOICES = (
    ('img', 'Image'),
    ('doc', 'Document'),
)


from django.views.generic import ListView, CreateView, View
from django.core.urlresolvers import reverse, reverse_lazy
from django.contrib.auth.decorators import user_passes_test
from django.http import QueryDict, HttpResponse
import json

try:
    from urllib.parse import urlsplit, urlunsplit
except ImportError:     # Python 2
    from urlparse import urlsplit, urlunsplit


from .models import Asset
from .forms import AssetForm
from .constants import MEDIABROWSER_PAGE_SELECTOR_URL, MEDIABROWSER_USER_PASSES_TEST

_ = lambda x:x

auth_required = user_passes_test(MEDIABROWSER_USER_PASSES_TEST)

# TODO: need to keep URL params
class StickyGetParamsMixin(object):
    """ Mixin that persists GET params between requests. """
    # This is primarily required because file upload form is
    # not directly ajaxable and file APIs are not yet fully
    # supported by all those explorer runts out there
    
    def get_success_url(self):
        """ Updates success URL with GET params. """
        url = super(StickyGetParamsMixin, self).get_success_url()
        scheme, netloc, path, query, fragment = urlsplit(url)
        get_params = QueryDict(query, mutable=True)
        get_params.update(self.request.GET)
        return urlunsplit((scheme, netloc, path, get_params.urlencode(), fragment))
    
    def get_context_data(self, *args, **kwargs):
        """ Adds QUERY_STRING to context variable. """
        context = super(StickyGetParamsMixin, self).get_context_data(*args, **kwargs)
        context['QUERY_STRING'] = self.request.GET.urlencode()
        context['page_selector_url'] = MEDIABROWSER_PAGE_SELECTOR_URL
        return context
    

class JsonResponseMixin(object):
    def response(self, data):
        return HttpResponse(json.dumps(data), content_type="application/json")


class AssetTypeMixin(object):
    asset_type = None # set by subclass

    def get_context_data(self, *args, **kwargs):
        context = super(AssetTypeMixin, self).get_context_data(*args, **kwargs)
        context['asset_type'] = getattr(self, 'context_asset_type', self.asset_type)
        return context


class BaseAssetListView(StickyGetParamsMixin, AssetTypeMixin, ListView):
    template_name = "mediabrowser/list.html"
    model = Asset
    context_object_name = "assets"
    
    def get_queryset(self):
        return Asset.objects.filter(type=self.asset_type)


class ImageListView(BaseAssetListView):
    asset_type = "img"

image_list_view = auth_required(ImageListView.as_view())


class ImageLinkListView(BaseAssetListView):
    """ View to browse images in the link context. """
    asset_type = "img"
    context_asset_type = "doc"

image_as_document_list_view = auth_required(ImageLinkListView.as_view())


class DocumentListView(BaseAssetListView):
    asset_type = "doc"

document_list_view = auth_required(DocumentListView.as_view())

class BaseAssetAddView(StickyGetParamsMixin, CreateView):
    model = Asset
    form_class = AssetForm
    template_name = "mediabrowser/add.html"
    template_name_success = "mediabrowser/add_success.html"
    
    def form_valid(self, form):
        """ Renders response that inserts asset into caller's editor. """
        self.object = form.save(commit=False)
        self.object.uploaded_by = unicode(self.request.user)
        self.object.save()
        
        return self.response_class(
            request = self.request,
            template = self.template_name_success,
            context = {"asset":self.object}
        )


class ImageAddView(AssetTypeMixin, BaseAssetAddView):
    asset_type = "img"
    
image_add_view = auth_required(ImageAddView.as_view())


class DocumentAddView(AssetTypeMixin, BaseAssetAddView):
    asset_type = "doc"
    
document_add_view = auth_required(DocumentAddView.as_view())



class AssetDeleteView(JsonResponseMixin, View):
    def post(self, request):
        return self.delete(request)
    
    def delete(self, request):
        try:
            asset_id = int(request.POST.get("asset_id"))
        except ValueError:
            return self.response({"status":"error", "message":_("Please select a file to be deleted.")})
        try:
            asset = Asset.objects.get(id=asset_id)
        except Asset.DoesNotExist:
            return self.response({"status":"error", "message":_("Invalid asset id.")})
        asset.delete()
        return self.response({"status":"ok", "id":asset_id})

asset_delete_view = auth_required(AssetDeleteView.as_view())




#!/usr/bin/env python

from setuptools import setup
import mediabrowser


setup(
    name='mediabrowser',
    version=mediabrowser.VERSION,
    description='Django media browser for WYSIWYG HTML editor',
    install_requires=["easy_thumbnails"],
    author='Sergiy Kuzmenko',
    author_email='sergiy@kuzmenko.org',
    url='https://github.com/shelldweller/mediabrowser',
    packages=['mediabrowser'],
    licence="MIT",
    package_data={"mediabrowser":[
        "README.md"
        "static/mediabrowser/css/smoothness/images/*",
        "static/mediabrowser/css/smoothness/*.css",
        "static/mediabrowser/css/*.css",
        "static/mediabrowser/img/*.png",
        "static/mediabrowser/js/*.js",
        "templates/mediabrowser/includes/*.html",
        "templates/mediabrowser/*.html",
        "tests/test-data/*.png",
        ]},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Plugins",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Topic :: Multimedia",
        "Topic :: Multimedia :: Graphics",
        "License :: OSI Approved :: MIT License",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)